# -*- coding: utf-8 -*-
"""
Sistema de Consulta Pública

Copyleft (@) 2017 CENDITEL nodo Mérida - https://planificacion.cenditel.gob.ve/trac/wiki/ModeladoTopicos_2017
"""
## @package base.functions
#
# Clases genéricas de la consulta
# @author Rodrigo Boet (rboet at cenditel.gob.ve)
# @author <a href='http://www.cenditel.gob.ve'>Centro Nacional de Desarrollo e Investigación en Tecnologías Libres
# (CENDITEL) nodo Mérida - Venezuela</a>
# @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
# @version 1.0
from __future__ import unicode_literals
from django.http import HttpResponseServerError
from django.contrib.auth.models import User
from consulta.models import Consulta,TipoPregunta, Pregunta
from participacion.models import RespuestaAbierta
from procesamiento.models import Procesamiento
from consulta_publica.settings import PROCESAMIENTO_PATH
from .models import Entidad, Municipio, Parroquia
from users.models import Perfil
import requests
import copy
import os
import shutil

def cargar_tipo_pregunta():
    """!
    Función que permite cargar los tipos de preguntas que existen

    @author Rodrigo Boet (rboet at cenditel.gob.ve)
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 15-02-2017
    @return Devuelve una tupla con los tipos de pregunta
    """

    lista = ('', 'Seleccione...'),

    try:
        for tipo_pregunta in TipoPregunta.objects.all():
            lista += (tipo_pregunta.id, tipo_pregunta.tipo),
    except Exception as e:
        pass

    return lista

def cargar_consulta():
    """!
    Función que permite cargar las consultas

    @author Rodrigo Boet (rboet at cenditel.gob.ve)
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 22-02-2017
    @return Devuelve una tupla con las consultas
    """

    lista = ('', 'Seleccione...'),

    try:
        for consulta in Consulta.objects.all():
            lista += (consulta.id, consulta.nombre_consulta),
    except Exception as e:
        pass

    return lista


def cargar_ministerios():
    """!
    Función que permite cargar los ministerios

    @author Antonio Araujo (aaraujo at cenditel.gob.ve)
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 15-03-2017
    @return Devuelve una tupla con los ministerios
    """

    # TODO: especificar la URL del servicio a traves de una configuracion y gestion
    # de error en solicitud http
    url =  'http://192.168.12.152:8080/services_sipes/entes_planificadores/views/servicios_webs?display_id=page_1'
    r = requests.get(url)

    if not r.status_code == 200:
        print "ERROR:"
        return ('', 'ocurrio error...'),

    data = r.json()
    # tupla de entes
    tuplaNameEntity = ('', 'Ministerios '),
    uniqueEntities = []
    nidEntities = []

    for dicc in data:
        tmpEntity = dicc['name_ente']
        tmpNidEntity = dicc['nid']
        if  tmpEntity.startswith('Ministerio') and  tmpEntity not in uniqueEntities:
            uniqueEntities.append(tmpEntity)
            nidEntities.append(tmpNidEntity)
        else:
            continue

    for i in range(0, len(uniqueEntities)) :
        tuplaNameEntity += (nidEntities[i], uniqueEntities[i]),

    return tuplaNameEntity


def cargar_vicepresidencias():
    """!
    Función que permite cargar las vicepresidencias

    @author Antonio Araujo (aaraujo at cenditel.gob.ve)
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 15-03-2017
    @return Devuelve una tupla con las vicepresidencias
    """

    # TODO: especificar la URL del servicio a traves de una configuracion y gestion
    # de error en solicitud http
    url =  'http://192.168.12.152:8080/services_sipes/entes_planificadores/views/servicios_webs?display_id=page_1'
    r = requests.get(url)
    if not r.status_code == 200:
        print "ERROR:"
        return ('', 'ocurrio error...'),

    data = r.json()

    # tupla de sectores
    tuplaSector = ('', 'Vicepresidencias'),
    uniqueSectors = []
    nidEntities = []

    for dicc in data:
        tmpSector = dicc['sector']
        tmpNidEntity = dicc['nid']
        if tmpSector.startswith('Vicepresidencia') and tmpSector not in uniqueSectors:
            uniqueSectors.append(tmpSector)
            nidEntities.append(tmpNidEntity)
        else:
            continue

    for i in range(0, len(uniqueSectors)) :
        tuplaSector += (nidEntities[i], uniqueSectors[i]),

    return tuplaSector


def cargar_ambitos():
    """!
    Función que permite cargar los ambitos

    @author Antonio Araujo (aaraujo at cenditel.gob.ve)
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 22-03-2017
    @return Devuelve una tupla con los ambitos
    """

    # TODO: especificar la URL del servicio a traves de una configuracion y gestion
    # de error en solicitud http
    url = 'http://192.168.12.152:8080/services_sipes/entes_planificadores/views/lista_ambitos_y_sectores?display_id=ambitos'
    r = requests.get(url)
    if not r.status_code == 200:
        print "ERROR:"
        return ('', 'ocurrio error...'),

    data = r.json()

    tuplaScope = ('', 'Seleccione...'),

    for dicc in data:
        if dicc['parent_id'] == '0' :
            tuplaScope += (dicc['id'], dicc['nombre']),

    return tuplaScope

def cargar_sectores():
    """!
    Función que permite cargar los sectores

    @author Antonio Araujo (aaraujo at cenditel.gob.ve)
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 22-03-2017
    @return Devuelve una tupla con los ambitos
    """

    # TODO: especificar la URL del servicio a traves de una configuracion y gestion
    # de error en solicitud http
    url = 'http://192.168.12.152:8080/services_sipes/entes_planificadores/views/lista_ambitos_y_sectores?display_id=ambitos'
    r = requests.get(url)
    if not r.status_code == 200:
        print "ERROR:"
        return ('', 'ocurrio error...'),

    data = r.json()
    tuplaSector = ('', 'Seleccione...'),

    for dicc in data:
        if dicc['parent_id'] != '0' :
            tuplaSector += (dicc['parent_id'], dicc['nombre']),
    return tuplaSector

def cargar_procesamiento(pk=None):
    """!
    Función que permite cargar los procesamientos

    @author Rodrigo Boet (rboet at cenditel.gob.ve)
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 28-03-2017
    @return Devuelve una tupla con los perfiles de procesamiento
    """

    lista = ('', 'Seleccione...'),

    Pro = Procesamiento.objects.filter(consulta_id=pk) if pk else Procesamiento.objects
    try:
        for procesamiento in Pro.all():
            lista += (procesamiento.id, procesamiento.procesamiento_dir),
    except Exception as e:
        pass

    return lista


def dump_exception():
    """!
    Función para captar los errores e imprimirlos

    @author Jorge Redondo (jredondo at cenditel.gob.ve)
    @copyright GNU/GPLv2
    @param request <b>{object}</b> Objeto que mantiene la peticion
    @return Retorna una respuesta http con el error
    """
    import sys,traceback
    exc_type, exc_value, exc_traceback = sys.exc_info()
    print "*** print_tb:"
    traceback.print_tb(exc_traceback, limit=1, file=sys.stdout)
    print "*** print_exception:"
    traceback.print_exception(exc_type, exc_value, exc_traceback, limit=2, file=sys.stdout)
    return HttpResponseServerError(str(exc_value))



def createPreprocessingFiles(id):
    """!
    Crea los archivos necesarios del preprocesamiento a partir de las respuestas
    abiertas de una consulta.

    La función creará el directorio consulta_publica/static/procesamiento_files/nombre_consulta/orig
    con un conjunto de archivos de texto que representan cada una de las repuestas
    abiertas de la consulta. Por ejemplo:
    .
    miconsulta
    |---orig
        |--- respuesta_5.txt
        |--- respuesta_6.txt

    En caso de existir el directorio al momento de ejecutarse esta función será
    borrado.

    @author Antonio Araujo  (aaraujo at cenditel.gob.ve)
    @copyright GNU/GPLv2
    @param id Identificador de la consulta para obtener respuestas abiertas
    """

    try:
        # obtener la consulta con id pasado como argumento
        consulta = Consulta.objects.get(id=id)
        
        ## Se crea un nombre sin espacios en blanco
        nombre = "_".join(consulta.nombre_consulta.split(" "))

        # crear un directorio con el nombre de la consulta en consulta_publica/static/procesamiento_files/
        processingDirectory = PROCESAMIENTO_PATH + '/' + nombre + '/orig'

        # borrar directorio en el caso de existir
        if os.path.isdir(processingDirectory) :
            shutil.rmtree(processingDirectory)
        os.makedirs(processingDirectory)

        # obtener las preguntas
        for pregunta in Pregunta.objects.filter(consulta_id=consulta.id).all() :
            if pregunta.tipo_pregunta_id==5 :
                # ubicar la respuesta
                for respuesta in RespuestaAbierta.objects.filter(pregunta_id=pregunta.id).all():
                    # escribir archivo a disco
                    filePath = processingDirectory + '/' + 'respuesta_' + str(respuesta.id)+".txt"
                    fo = open(filePath, "w")
                    fo.write(respuesta.texto_respuesta.encode('utf8'));
                    fo.close()
        
        return True

    except Exception as e:
        print "La consulta con identificado "+ str(id) + " no existe"
        print e
        return False
        pass
    
def validate_dir(pk):
    """!
    Función que permite cargar las consultas con directorios creados

    @author Rodrigo Boet (rboet at cenditel.gob.ve)
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 17-04-2017
    @param pk {int} Recibe id de la consulta
    @return Devuelve una tupla con las consultas que tienen directorios
    """
    
    consulta = Consulta.objects.filter(pk=pk)
    if consulta:
        consulta = consulta.get()
        nombre = "_".join(consulta.nombre_consulta.split(" "))
        consulta_dir = PROCESAMIENTO_PATH+"/"+nombre
        if os.path.exists(consulta_dir):
            return True
        return False
    else:
        return False

def cargar_entidad():
    """!
    Función que permite cargar todas las entidades

    @author Rodrigo Boet (rboet at cenditel.gob.ve)
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 20-04-2017
    @return Devuelve una tupla con las entidades
    """

    lista = ('', 'Seleccione...'),

    try:
        for entidad in Entidad.objects.all():
            lista += (entidad.codigo, entidad.nombre),
    except Exception as e:
        pass

    return lista


def cargar_municipios():
    """!
    Función que permite cargar todas los municipios

    @author Rodrigo Boet (rboet at cenditel.gob.ve)
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 20-04-2017
    @return Devuelve una tupla con los municipios
    """

    lista = ('', 'Seleccione...'),

    try:
        for municipio in Municipio.objects.all():
            lista += (municipio.codigo, municipio.nombre),
    except Exception as e:
        pass

    return lista


def cargar_parroquias():
    """!
    Función que permite cargar todas las parroquias

    @author Rodrigo Boet (rboet at cenditel.gob.ve)
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 20-04-2017
    @return Devuelve una tupla con las parroquias
    """

    lista = ('', 'Seleccione...'),

    try:
        for parroquia in Parroquia.objects.all():
            lista += (parroquia.codigo, parroquia.nombre),
    except Exception as e:
        pass

    return lista


def validate_cedula(cedula):
    """!
    Función que permite validar la cedula

    @author Rodrigo Boet (rboet at cenditel.gob.ve)
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 20-04-2017
    @param cedula {str} Recibe el número de cédula
    @return Devuelve verdadero o falso
    """
    
    cedula = Perfil.objects.filter(cedula=cedula)
    if cedula:
        return True
    else:
        return False
    
def validate_email(email):
    """!
    Función que permite validar la cedula

    @author Rodrigo Boet (rboet at cenditel.gob.ve)
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 20-04-2017
    @param cedula {str} Recibe el número de cédula
    @return Devuelve verdadero o falso
    """
    
    email = User.objects.filter(email=email)
    if email:
        return True
    else:
        return False
